@@ -1,6 +1,7 @@
 import decimal
 import json
 import os
+import sys
 import statistics
 import time
 import uuid
@@ -16,17 +17,24 @@ from starkware_helpers.signature import get_price_msg, sign
 from aptos_sdk.account import Account
 from aptos_sdk.bcs import Serializer
 
+import logging
+from watchtower import CloudWatchLogHandler
+
 import hashlib
 
 load_dotenv()
 
+
 def setup_lib(
     oracle_name,
     starkex_private_key,
     evm_address,
     evm_private_key,
     aptos_address = None,
-    aptos_private_key = None
+    aptos_private_key = None,
+    enable_cloudwatch_logs = False,
+    cloudwatch_log_group = None,
+    cloudwatch_log_stream = None,
 ):
     '''
     Setup constants for the lib to use.
@@ -37,9 +45,11 @@ def setup_lib(
     :param evm_private_key: private key used to generate evm signed price
     :param aptos_address: aptos address used for signing
     :param aptos_private_key: private key used to generate aptos signed price
+    :param cloudwatch_log_group: AWS Cloudwatch log group to send messages to
+    :param cloudwatch_log_stream: AWS Cloudwatch log stream (within cloudwatch_log_group) to send messages to
     '''
 
-    global ORACLE_NAME, ORACLE_NAME_HEX, ORACLE_NAME_INT, STARKEX_PRIVATE_KEY, EVM_ADDRESS, EVM_PRIVATE_KEY, APTOS_ADDRESS, APTOS_PRIVATE_KEY
+    global ORACLE_NAME, ORACLE_NAME_HEX, ORACLE_NAME_INT, STARKEX_PRIVATE_KEY, EVM_ADDRESS, EVM_PRIVATE_KEY, APTOS_ADDRESS, APTOS_PRIVATE_KEY, STABLECOIN_THRESHOLD, LOG_PARAMS
 
     ORACLE_NAME = oracle_name
     ORACLE_NAME_HEX = ORACLE_NAME.encode("utf-8").hex()
@@ -49,6 +59,40 @@ def setup_lib(
     EVM_PRIVATE_KEY = evm_private_key
     APTOS_ADDRESS = aptos_address
     APTOS_PRIVATE_KEY = aptos_private_key
+    STABLECOIN_THRESHOLD = 0.25 # maximum allowed deviation from parity for stablecoins
+
+    init_logger(enable_cloudwatch_logs, cloudwatch_log_group, cloudwatch_log_stream)
+
+
+def init_logger(enable_cloudwatch_logs, cloudwatch_log_group, cloudwatch_log_stream):
+    """
+    Init the global root logger and set default console handler and, if applicable, AWS Cloudwatch.
+
+    :param enable_cloudwatch_logs: Whether or not to enable sending logs to Cloudwatch
+    :param cloudwatch_log_group: AWS Cloudwatch log group to send messages to
+    :param cloudwatch_log_stream: AWS Cloudwatch log stream (within cloudwatch_log_group) to send messages to
+    """
+    global root_logger
+
+    root_logger = logging.getLogger()
+    root_logger.setLevel(logging.INFO)
+
+    stream_handler = logging.StreamHandler(sys.stdout)
+    stream_handler.setLevel(logging.DEBUG)
+    root_logger.addHandler(stream_handler)
+
+    if enable_cloudwatch_logs and bool(cloudwatch_log_group) and bool(cloudwatch_log_stream):
+        cw_handler = CloudWatchLogHandler(
+            log_group_name=cloudwatch_log_group,
+            log_stream_name=cloudwatch_log_stream,
+        )
+        cw_handler.setLevel(logging.INFO)
+        root_logger.addHandler(cw_handler)
+        root_logger.info(f"Cloudwatch logs enabled: log_group = {cloudwatch_log_group}, log_stream = {cloudwatch_log_stream}")
+    else:
+        root_logger.info("Cloudwatch logs not enabled")
+
+    root_logger.info("Oracle started")
 
 
 def starkex_sign(external_asset_padded, timestamp, external_price):
@@ -182,7 +226,7 @@ async def send(stork_endpoint, asset, starkex_signed_price, evm_signed_price, ap
     if aptos_signed_price:
         message["aptos_signed_price"] = aptos_signed_price
 
-    print(f"Sending message for asset {asset} to stork, {correlation_id}.")
+    root_logger.debug(f"Sending message for asset {asset} to stork, {correlation_id}.")
     await stork_endpoint.send(json.dumps(message))
 
 
@@ -260,7 +304,7 @@ def parse_env_vars(var_name):
     """
     This function takes a given environment variable name, and parses it from the .env file
 
-    :param var_name_hex: given key of environment variable, which corresponds to a hex value stored in the .env file
+    :param var_name: given key of environment variable, which corresponds to a hex value stored in the .env file
     :returns parsed_var: returns env var converted into hexadecimal
     """
     parsed_var = hex(int(os.getenv(var_name), 16))
@@ -307,3 +351,24 @@ def can_skip_update(current_asset, median_price, last_updates, exchange=None, ex
         return True
 
     return False
+
+def convert_to_usd(price, current_quote, exchange_rates):
+    """
+    This function converts price from current_quote to USD based on exchange_rates, or returns None.
+
+    :param price: price of current asset
+    :param current_quote: the quote currency of the current asset
+    :param exchange_rates: dict containing exchange rates for various assets
+    :returns float: returns the converted price if data is valid, or None if data is invalid
+    """
+    if current_quote in exchange_rates:
+        exchange_rate = exchange_rates[current_quote]
+
+        # this is a sanity check to make sure the exchange rate is not too far off
+        if abs(exchange_rate - 1) > STABLECOIN_THRESHOLD:
+            return None
+
+        return float(price) * exchange_rate
+    else:
+        root_logger.info(f"No exchange rate for {current_quote} to USD")
+        return None
\ No newline at end of file
